/**---------------------------------------------------------------------
***  
***  file:    xmfonts.c
***
***  project:  Motif Widgets example programs
***
***  description:  This program creates a button for every font in
***      /usr/lib/X11/fonts.  When a button is selected,
***      a text sample is displayed using the font.
***  
***  
***      (c) Copyright 1989 by Open Software Foundation, Inc.
***          All Rights Reserved.
***
***      (c) Copyright 1989 by Hewlett-Packard Company.
***
***
***  defaults:  xmfonts.c depends on these defaults:
#
*keyboardFocusPolicy:    POINTER
*XmScrolledWindow.height:  432
*XmScrolledWindow.width:    690
*menu_bar*background:    #58f
*menu_bar*fontList:  vr-20
*XmPushButtonGadget*fontList:  vr-20
***-------------------------------------------------------------------*/

/*-------------------------------------------------------------
**  Include Files
*/
#include <stdio.h>
/* #include <string.h> */
#include <sys/types.h>
#ifdef BSD
#include <sys/dir.h>
#else
#include <dirent.h>
#endif

#include <Xm/BulletinB.h>
#include <Xm/CascadeB.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/MainW.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrollBar.h>
#include <Xm/ScrolledW.h>


/* information for 'what' */
char ident1[]="@(#)xmfonts Version A.00.28 17 December 1990";

/*-------------------------------------------------------------
**  Forward Declarations
*/

void main ();      /*  main logic for application  */

Widget CreateApplication ();  /*  create main window    */
Widget CreateFontSample ();  /*  create font display window  */
Widget CreateHelp ();    /*  create help window    */
Widget CreateVersion ();    /* create version window */

void VersionCB ();    /* callback for version window */
void SelectFontCB ();    /*  callback for font buttons  */
void HelpCB ();      /*  callback for help button  */
void QuitCB ();      /*  callback for quit button  */



/*-------------------------------------------------------------
**  Global Variables
*/

#define MAX_ARGS 20
#define FONT_DIR_NAME "/usr/lib/X11/fonts/misc"
#define TITLE_STRING "X Font Sampler"

static XmStringCharSet charset = (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;

/*-------------------------------------------------------------
**  main    - main logic for application
*/
void main (argc,argv) 
  unsigned int  argc;
  char    **argv;
{
  Display    *display;  /*  Display    */
  Widget    app_shell;  /*  ApplicationShell  */
  Widget    main_window;  /*  MainWindow    */
  ArgList      args;    /* argument list */
  XtAppContext app_context; /* Application Context */


  /*  Initialize toolkit */
  app_shell = XtAppInitialize(&app_context, "XMfonts", NULL, 0, &argc,
  argv, NULL, args, 0);

  main_window = CreateApplication (app_shell);
  XtRealizeWidget (app_shell);

  /*  Get and dispatch events. */

  XtAppMainLoop (app_context);
}

/*-------------------------------------------------------------
**  CreateApplication  - create main window
*/
Widget CreateApplication (parent) 
Widget    parent;    /*  parent widget  */
{
  Widget    main_window;  /*  MainWindow    */
  Widget    menu_bar;  /*  MenuBar    */
  Widget    menu_pane;  /*  MenuPane    */
  Widget    cascade;  /*  CascadeButton  */
  Widget    frame;    /*  Frame    */
  Widget    swindow;  /*  ScrolledWindow  */
  Widget    row_column;  /*  RowColumn    */
  Widget    button;    /*  PushButtonGadget  */
  Widget    hsb, vsb;  /*  ScrollBars    */

  Arg    args[MAX_ARGS];  /*  arg list    */
  register int  n;    /*  arg count    */

  DIR    *dirp;    /*  directory pointer  */
#ifdef BSD
  struct direct  *item;    /*  entry in directory  */
#else
  struct dirent  *item;    /*  entry in directory  */
#endif
  char    name[15];  /*  name string    */
  int    len;    /*  string length  */

  XmString  label_string;


  /*  Create MainWindow.
  */
  n = 0;
  main_window = XmCreateMainWindow (parent, "main1", args, n);
  XtManageChild (main_window);

  /*  Create MenuBar in MainWindow.
  */
  n = 0;
  menu_bar = XmCreateMenuBar (main_window, "menu_bar", args, n); 
  XtManageChild (menu_bar);

  /*  Create "Exit" PulldownMenu.
  */
  n = 0;
  menu_pane = XmCreatePulldownMenu (menu_bar, "menu_pane", args, n);

  n = 0;
  button = XmCreatePushButton (menu_pane, "Quit", args, n);
  XtManageChild (button);
  XtAddCallback (button, XmNactivateCallback, QuitCB, NULL);

  n = 0;
  XtSetArg (args[n], XmNsubMenuId, menu_pane);  n++;
  cascade = XmCreateCascadeButton (menu_bar, "Exit", args, n);
  XtManageChild (cascade);


  /*  Create "Help" pulldown menu.
  */

  n = 0;
  menu_pane = XmCreatePulldownMenu (menu_bar, "menu_pane", args, n);

  n = 0;
  button = XmCreatePushButton (menu_pane, "Help", args, n);
  XtManageChild (button);
  XtAddCallback (button, XmNactivateCallback, HelpCB, NULL);

  n = 0;
  button = XmCreatePushButton (menu_pane, "Program Version", args, n);
  XtManageChild (button);
  XtAddCallback (button, XmNactivateCallback, VersionCB, NULL);

  n = 0;
  XtSetArg (args[n], XmNsubMenuId, menu_pane);  n++;
  cascade = XmCreateCascadeButton (menu_bar, "Help", args, n);
  XtManageChild (cascade);

  n = 0;
  XtSetArg (args[n], XmNmenuHelpWidget, cascade);  n++;
  XtSetValues (menu_bar, args, n);


  /*  Create Frame MainWindow and ScrolledWindow in Frame.
  */
  n = 0;
  XtSetArg (args[n], XmNmarginWidth, 2);  n++;
  XtSetArg (args[n], XmNmarginHeight, 2);  n++;
  XtSetArg (args[n], XmNshadowThickness, 1);  n++;
  XtSetArg (args[n], XmNshadowType, XmSHADOW_OUT);  n++;
  frame = XmCreateFrame (main_window, "frame", args, n);
  XtManageChild (frame);

  n = 0;
  XtSetArg (args[n], XmNscrollBarDisplayPolicy, XmAS_NEEDED);  n++;
  XtSetArg (args[n], XmNscrollingPolicy, XmAUTOMATIC);  n++;
  swindow = XmCreateScrolledWindow (frame, "swindow", args, n);
  XtManageChild (swindow);


  /*  Create RowColumn in ScrolledWindow to manage buttons.
  */
  n = 0;
  XtSetArg (args[n], XmNpacking, XmPACK_COLUMN);  n++;
  XtSetArg (args[n], XmNnumColumns, 5);  n++;
  row_column = XmCreateRowColumn (swindow, "row_column", args, n);
  XtManageChild (row_column);


  /*  Set MainWindow areas and add tab groups
  */
  XmMainWindowSetAreas (main_window, menu_bar, NULL, NULL, NULL,
      frame);
  n = 0;
  XtSetArg (args[n], XmNhorizontalScrollBar, &hsb);  n++;
  XtSetArg (args[n], XmNverticalScrollBar, &vsb);  n++;
  XtGetValues (main_window, args, n);
  XmAddTabGroup (row_column);
  if (hsb)
    XmAddTabGroup (hsb);
  if (vsb)
    XmAddTabGroup (vsb);


  /*  Create a PushButton widget for each font.
  */
  /*  open the font directory  */
  dirp = opendir (FONT_DIR_NAME);
  /*  read one entry each time through the loop  */
  for (item = readdir (dirp);  item != NULL;  item = readdir (dirp)) 
  {
    len =  (strlen (item -> d_name));
    if (item -> d_name[len-1] != 'Z') {  /* Last letter not Z */
      /*  discard entries that don't end in ".xxx"  */
      if ((len < 5) || (item -> d_name[len-4] != '.')) continue;
      /*  copy the name (except extension) from the entry  */
      strncpy (name, item -> d_name, len-4);
      name[len-4] = '\0';
	}
    else {
      /*  discard entries that don't end in ".xxx.Z"  */
      if ((len < 7) || (item -> d_name[len-6] != '.')) continue;
      /*  copy the name (except extension) from the entry  */
      strncpy (name, item -> d_name, len-6);
      name[len-6] = '\0';
    }
  /*  create PushButton in RowCol  */
    n = 0;
    label_string = XmStringCreateLtoR(name, charset);
    XtSetArg (args[n], XmNlabelString, label_string);  n++;
    button = XmCreatePushButtonGadget (row_column, name, args, n);
    XtManageChild (button);
    XtAddCallback (button, XmNarmCallback, SelectFontCB, NULL);
    XmStringFree (label_string);
  }

  /*  Return MainWindow.
  */
  return (main_window);
}

/*-------------------------------------------------------------
**  CreateFontSample  - create font display window
*/
Widget CreateFontSample (parent) 
Widget    parent;    /*  parent widget  */
{
  Widget    message_box;    /*  MessageBox Dialog  */
  Widget    button;
  Arg    args[MAX_ARGS];    /*  arg list    */
  register int  n;      /*  arg count    */
  
  char    *name = NULL;    /*  font name    */
  XFontStruct  *font = NULL;    /*  font pointer  */
  XmFontList  fontlist = NULL;  /*  fontlist pointer  */
  static char  message[BUFSIZ];  /*  text sample    */
  XmString  name_string = NULL;
  XmString  message_string = NULL;
  XmString  button_string = NULL;

  
  /*  Get font name.
  */
  n = 0;
  XtSetArg (args[n], XmNlabelString, &name_string); n++;
  XtGetValues (parent, args, n);
  XmStringGetLtoR (name_string, charset, &name);


  /*  Load font and generate message to display.
  */
  if (name)
    font = XLoadQueryFont (XtDisplay (XtParent (parent)), name);
  if  (!font)
    sprintf (message, "Unable to load font: %s\0", name);
  else
  {
    fontlist = XmFontListCreate (font, charset);
    sprintf (message, "\
This is font %s.\n\
The quick brown fox jumps over the lazy dog.\0", name);
  }
  message_string = XmStringCreateLtoR (message, charset);
  button_string = XmStringCreateLtoR ("Close", charset);


  /*  Create MessageBox dialog.
  */
  n = 0;
  if (fontlist) 
  {
    XtSetArg (args[n], XmNlabelFontList, fontlist);  n++;
  }
  XtSetArg (args[n], XmNdialogTitle, name_string);  n++;
  XtSetArg (args[n], XmNokLabelString, button_string);  n++;
  XtSetArg (args[n], XmNmessageString, message_string);  n++;
  message_box = XmCreateMessageDialog (XtParent (XtParent(parent)), "fontbox",
        args, n);

  button = XmMessageBoxGetChild (message_box, XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild (button);
  button = XmMessageBoxGetChild (message_box, XmDIALOG_HELP_BUTTON);
  XtUnmanageChild (button);

  /*  Free strings and return MessageBox.
  */
  if (fontlist) XmFontListFree (fontlist);
  if (name_string) XtFree (name_string);
  if (message_string) XtFree (message_string);
  if (button_string) XtFree (button_string);
  return (message_box);
}

/*-------------------------------------------------------------
**  VersionCB      - callback for help button
*/
void VersionCB (w, client_data, call_data) 
Widget    w;    /*  widget id    */
XtPointer    client_data;  /*  data from application   */
XtPointer    call_data;  /*  data from widget class  */
{
  Widget    message_box;    /*  MessageBox    */


  /*  Create message window. */
  message_box = CreateVersion (w);


  /*  Display version window. */
  XtManageChild (message_box);
}

/*-------------------------------------------------------------
**  CreateVersion    - create version window
*/
Widget CreateVersion (parent) 
  Widget    parent;    /*  parent widget  */
{
  Widget    button;
  Widget    message_box;  /*  Message Dialog   */
  Arg    args[MAX_ARGS];  /*  arg list    */
  register int  n;    /*  arg count    */

  static char  message[BUFSIZ];  /*  help text  */
  XmString  title_string = NULL;
  XmString  message_string = NULL;
  XmString  button_string = NULL;
  char text4[50];


  strcpy (text4, &ident1[4]);
  /*  Generate message to display. */
  sprintf (message, "This is %s \0",text4);

  /* Create the compound strings */
  message_string = XmStringCreateLtoR (message, charset);
  button_string = XmStringCreateLtoR ("Close", charset);
  title_string = XmStringCreateLtoR ("diskuse help", charset);


  /*  Create MessageBox dialog. */
  n = 0;
  XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
  XtSetArg (args[n], XmNokLabelString, button_string);  n++;
  XtSetArg (args[n], XmNmessageString, message_string);  n++;
  message_box = XmCreateMessageDialog (parent, "versionbox", args, n);

  button = XmMessageBoxGetChild (message_box, XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild (button);
  button = XmMessageBoxGetChild (message_box, XmDIALOG_HELP_BUTTON);
  XtUnmanageChild (button);


  /*  Free strings and return MessageBox. */
  if (title_string) XtFree (title_string);
  if (message_string) XtFree (message_string);
  if (button_string) XtFree (button_string);
  return (message_box);
}

/*-------------------------------------------------------------
**  CreateHelp    - create help window
*/
Widget CreateHelp (parent) 
  Widget    parent;    /*  parent widget  */
{
  Widget    button;
  Widget    message_box;  /*  Message Dialog  */
  Arg    args[MAX_ARGS];  /*  arg list    */
  register int  n;    /*  arg count    */

  static char  message[BUFSIZ];  /*  help text  */
  XmString  title_string = NULL;
  XmString  message_string = NULL;
  XmString  button_string = NULL;



  /*  Generate message to display.
  */
  sprintf (message, "\
These are buttons for the fonts in the X11 font directory specified \n\
in the program variable FONT_DIR_NAME.  The default directory is \n\
/usr/lib/X11/fonts/misc.  You can change this variable to any directory \n\
and then recompile the program. \n\
\n\
The button label is the name of the font.  When you select \n\
a button, a small window will display a sample of the font.  \n\n\
Press the 'Close' button to close a font window.  \n\
Select 'Quit' from the 'Exit' menu to exit this application.\0");
  message_string = XmStringCreateLtoR (message, charset);
  button_string = XmStringCreateLtoR ("Close", charset);
  title_string = XmStringCreateLtoR ("xmfonts help", charset);


  /*  Create MessageBox dialog.
  */
  n = 0;
  XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
  XtSetArg (args[n], XmNokLabelString, button_string);  n++;
  XtSetArg (args[n], XmNmessageString, message_string);  n++;
  message_box = XmCreateMessageDialog (parent, "helpbox", args, n);

  button = XmMessageBoxGetChild (message_box, XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild (button);
  button = XmMessageBoxGetChild (message_box, XmDIALOG_HELP_BUTTON);
  XtUnmanageChild (button);


  /*  Free strings and return MessageBox.
  */
  if (title_string) XtFree (title_string);
  if (message_string) XtFree (message_string);
  if (button_string) XtFree (button_string);
  return (message_box);
}

/*-------------------------------------------------------------
**  SelectFontCB    - callback for font buttons
*/
void SelectFontCB (w, client_data, call_data) 
Widget    w;    /*  widget id    */
XtPointer    client_data;  /*  data from application   */
XtPointer    call_data;  /*  data from widget class  */
{
  Widget    message_box;


  /*  Create font sample window.
  */
  message_box = CreateFontSample (w);


  /*  Display font sample window.
  */
  XtManageChild (message_box);
}

/*-------------------------------------------------------------
**  HelpCB      - callback for help button
*/
void HelpCB (w, client_data, call_data) 
Widget    w;    /*  widget id    */
XtPointer    client_data;  /*  data from application   */
XtPointer    call_data;  /*  data from widget class  */
{
  Widget    message_box;    /*  MessageBox    */


  /*  Create help window.
  */
  message_box = CreateHelp (w);


  /*  Display help window.
  */
  XtManageChild (message_box);
}

/*-------------------------------------------------------------
**  QuitCB      - callback for quit button
*/
void QuitCB (w, client_data, call_data) 
Widget    w;    /*  widget id    */
XtPointer    client_data;  /*  data from applicaiton   */
XtPointer    call_data;  /*  data from widget class  */
{


  /*  Terminate the application.
  */
  exit (0);
}
